package com.gic.haoban.manage.service.service;

import com.gic.api.base.commons.Page;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.manage.api.dto.StaffClerkRelationDTO;
import com.gic.haoban.manage.service.entity.MemberClerkChatConfig;
import com.gic.haoban.manage.service.entity.TabHaobanStaffClerkRelation;

import java.util.List;
import java.util.Set;

public interface StaffClerkRelationService {

	List<StaffClerkRelationDTO> listBindCode(String enterpriseId, Set<String> clerkCodeList);

	List<StaffClerkRelationDTO> listBindCodeByStaffId(List<String> enterpriseIdList, String staffId);

	/**
	 * 删除绑定
	 *
	 * @param clerkId
	 * @return
	 */
	boolean delBind(String clerkId, String optStaffId, int chanelCode);

	/**
	 * 删除绑定
	 *
	 * @param wxEnterpriseId 可以不传
	 * @param storeIds
	 * @return
	 */
	boolean delBindByStoreIds(String wxEnterpriseId, Set<String> storeIds, String optStaffId, int chanelCode);

	/**
	 * 删除绑定
	 *
	 * @param wxEnterpriseId 可以不传
	 * @param enterpriseId
	 * @param storeIds       解除除了这些门店之外的绑定的门店
	 * @return
	 */
	boolean delBindNotInStoreIds(String wxEnterpriseId, String enterpriseId, Set<String> storeIds, String optStaffId, int chanelCode);

	/**
	 * 绑定
	 *
	 * @param dto
	 * @return
	 */
	String bind(StaffClerkRelationDTO dto, String optStaffId, int chanelCode);

	/**
	 * 根据clerkId 更新数据
	 * @param dto
	 * @return
	 */
	boolean updateByClerkId(StaffClerkRelationDTO dto);

	/**
	 * 查询绑定根据clerkId
	 *
	 * @param clerkId
	 * @param wxEnterpriseId
	 * @return
	 */
	StaffClerkRelationDTO getBindByClerkId(String clerkId, String wxEnterpriseId);

	StaffClerkRelationDTO getByCodeAndEnterpriseId(String clerkCode, String enterpriseId);

	String insert(StaffClerkRelationDTO staffClerkRelation);

	int cleanStaffClerk(String wxEnterpriseId, List<String> staffIds);

	StaffClerkRelationDTO getOneBindCodeNoStatus(String enterpriseId, String clerkCode);

	StaffClerkRelationDTO getOneBindByStoreId(String staffId, String storeId);

	StaffClerkRelationDTO getByClerkId(String clerkId);

	StaffClerkRelationDTO getByClerkIdNoStatus(String clerkId);

	List<StaffClerkRelationDTO> listByClerkIds(List<String> clerkList);

	List<StaffClerkRelationDTO> listByStaffId(String wxEnterpriseId, String staffId);

	/**
	 * 保存
	 * @param memberClerkChatConfig
	 */
	void saveChatConfig(MemberClerkChatConfig memberClerkChatConfig);

	/**
	 * @param staffId
	 * @param memberId
	 * @return
	 */
	MemberClerkChatConfig getChatConfig(String staffId, String memberId);

	/**
	 * 根据企业查询
	 * @param wxEnterpriseId
	 * @return
	 */
	List<TabHaobanStaffClerkRelation> listByWxEnterpriseId(String wxEnterpriseId);

	Page<StaffClerkRelationDTO> pageByWxEnterpriseId(String wxEnterpriseId, BasePageInfo pageInfo);

	/**
	 * 根据门店获取门店的绑定列表
	 *
	 * @param wxEnterpriseId
	 * @param storeId
	 * @return
	 */
	List<StaffClerkRelationDTO> listBindByStoreId(String wxEnterpriseId, String storeId);

	/**
	 * 查询门店列表
	 *
	 * @param enterpriseId
	 * @return
	 */
	List<StaffClerkRelationDTO> listBindStoreIdByEnterpriseId(String enterpriseId);

	/**
	 * 查询所有正常导购
	 *
	 * @return {@link List<StaffClerkRelationDTO> }
	 * @author xuwenqian
	 * @date 2021-07-15 16:26:17
	 */
	List<StaffClerkRelationDTO> listAll();

	/**
	 * 查询有关联的员工id列表
	 *
	 * @param staffIdList 员工id列表
	 * @author: YongEn
	 * @return {@link Set<String>}
	 */
	List<String> listRelationsStaffId(Set<String> staffIdList);


	/**
	 * 更新用户id wx企业id
	 *
	 * @param wxEnterpriseId wx企业标识
	 * @return int
	 * @author mozhu
	 * @date 2022-06-30 16:21:06
	 */
	int updateUserIdByWxEnterpriseId(String wxEnterpriseId);
}
