package com.gic.haoban.manage.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.service.*;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.qo.QywxXcxSendMessageQo;
import com.gic.redis.data.util.GicRateLimiter;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.QywxNewUseridDTO;
import com.gic.wechat.api.dto.qywx.QywxXcxSendMessageDTO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RRateLimiter;
import org.redisson.api.RateIntervalUnit;
import org.redisson.api.RateType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * Created 2020/4/9.
 *
 * @author hua
 */
@RestController
@RequestMapping("/test")
public class TestController extends WebBaseController {


    private static final Logger logger = LoggerFactory.getLogger(TestController.class);

    private final ExecutorService pools = Executors.newFixedThreadPool(500);

    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;

    @Autowired
    private MemberUnionidRelatedApiService memberUnionidRelatedApiService;

    @Autowired
    private TestApiService testApiService;

    @Autowired
    private DepartmentApiService departmentApiService;

    @Autowired
    private MaidianLogApiService maidianLogApiService;

    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;

    @Autowired
    private ExternalClerkRelatedApiService externalClerkRelatedApiService;
    @Autowired
    private WxEnterpriseApiService wxEnterpriseApiService;
    @Autowired
    private QywxUserApiService qywxUserApiService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private WxApplicationApiService wxApplicationApiService;
    @Autowired
    private Config config;

    @RequestMapping("/send-message-test")
    public HaobanResponse testSendMessage(QywxXcxSendMessageQo qo) {
        QywxXcxSendMessageDTO messageDTO = EntityUtil.changeEntityByOrika(QywxXcxSendMessageDTO.class, qo);
        ArrayList<String> list = new ArrayList<>();
        list.add(qo.getUserId());

        messageDTO.setUserIds(list);
        boolean b = qywxSuiteApiService.sendMessage("ww9ede832a84b7ae5f",
                "ww2c34dc56739bb105", messageDTO);
        if (b) {
            return resultResponse(HaoBanErrCode.ERR_1);
        } else {
            return resultResponse(HaoBanErrCode.ERR_0);
        }
    }

    @RequestMapping("/clean-out")
    public HaobanResponse cleanMessage(String cid) {
        memberUnionidRelatedApiService.cleanByCid(cid);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/test-dubbo")
    public HaobanResponse dubbo(@RequestParam(defaultValue = "2000") Integer n, @RequestParam(defaultValue = "100") String t) {
        int i = n;
        while (i-- > 0) {
            int finalI = i;
            pools.execute(new Runnable() {
                @Override
                public void run() {
                    long start = System.currentTimeMillis();
                    testApiService.checkThread(finalI + "", Long.valueOf(t));
                    logger.info("时间:{}", System.currentTimeMillis() - start);
                }
            });
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/lock")
    public HaobanResponse lock(String name, Long time, int ty) {
        RedisUtil.lock(name, time, TimeUnit.SECONDS, 0L);
        if (ty == 0) {
            if (RedisUtil.isLocked(name)) {
                RedisUtil.unlock(name);
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/maidian-log")
    public HaobanResponse maidian(String hbversion, BasePageInfo pageInfo) {
        Page<MaidianLogDTO> page = maidianLogApiService.queryList(hbversion, pageInfo);
        return resultResponse(HaoBanErrCode.ERR_1, page);
    }

    @RequestMapping("/deal-slef-qywx-sys-department")
    public HaobanResponse dealSys1(String corpid, String secretVal, String dataId) {
        testApiService.listSelfQywxUser(corpid, secretVal, dataId);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/deal-all-qywx-sys-department")
    public HaobanResponse dealSys2(String corpid, String secretVal, String dataId) {
        testApiService.listAllQywxUser(corpid, secretVal, dataId);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/test-limiter")
    public HaobanResponse testLimiter(String key, String eid) {
        if (StringUtils.isAnyBlank(key, eid)) {
            return resultResponse(HaoBanErrCode.ERR_5);
        }
        GicRateLimiter instance = GicRateLimiter.getInstance("haoban-manage3-web");
        Long acquire = instance.acquire(key, eid);
        if (acquire == 0L) {
            return resultResponse(HaoBanErrCode.ERR_13);
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/init-history")
    public HaobanResponse initHistory(String wxEnterpriseId) {
        wxEnterpriseRelatedApiService.initHistoryEnterprise(wxEnterpriseId);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 获取到分表到哪个表内
     *
     * @param tableId 分表id
     * @param count   分表数量
     * @return
     */
    @RequestMapping("/test-table")
    public HaobanResponse testTable(String tableId, Integer count) {
        if (StringUtils.isBlank(tableId)) {
            return resultResponse(HaoBanErrCode.ERR_5);
        }
        if (count == null) {
            return resultResponse(HaoBanErrCode.ERR_5);
        }
        int i = tableId.hashCode() % count;
        return resultResponse(HaoBanErrCode.ERR_1, i);
    }

    /**
     * 设置迁移完成
     *
     * @param corpid
     * @param serviceCorpid
     * @return
     */
    @RequestMapping("/finishExternalUseridMigration")
    public HaobanResponse finishExternalUseridMigration(String corpid, String serviceCorpid) {
        boolean flag = qywxUserApiService.finishExternalUseridMigration(corpid, serviceCorpid);
        if (flag) {
            WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getEnterpriseBycorpId(corpid);
            if (wxEnterpriseDTO == null) {
                return resultResponse(HaoBanErrCode.ERR_10012);
            }
            wxEnterpriseApiService.agreeExternalUseridFlag(wxEnterpriseDTO.getWxEnterpriseId(), 1);
            return resultResponse(HaoBanErrCode.ERR_1);
        } else {
            return resultResponse(HaoBanErrCode.ERR_0);
        }
    }

    /**
     * 测试授权回调
     *
     * @param param
     * @return
     */
    @RequestMapping("/callbackReviceEnterprise")
    public HaobanResponse callbackReviceEnterprise(String param) {
        QywxCorpInfoSimpleDTO qywxCorpInfoSimpleDTO = JSON.parseObject(param, QywxCorpInfoSimpleDTO.class);
        wxEnterpriseApiService.callbackReviceEnterprise(qywxCorpInfoSimpleDTO);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 后台预留批量处理wx_user_id to wx_open_user_id
     *
     * @param corpid
     * @return
     */
    @RequestMapping("/useridToOpenuserid")
    public HaobanResponse useridToOpenuserid(String corpid) {
        WxApplicationDTO wxApplicationDTO = wxApplicationApiService.selectByCorpid(corpid);
        String wxEnterpriseId = wxApplicationDTO.getWxEnterpriseId();
        String siteId = wxApplicationDTO.getSiteId();
        // 限流器
        RRateLimiter rateLimiter = RedisUtil.getRedisClient().getRateLimiter("haoban:manage3:updateWxOpenUserId:limit");
        // 每1秒产生5个令牌
        rateLimiter.trySetRate(RateType.OVERALL, 5, 1, RateIntervalUnit.SECONDS);
        int pageNum = 1;
        while (true) {
            rateLimiter.acquire();
            //批量处理wx_user_id to wx_open_user_id
            Page<StaffDTO> staffDTOPageInfo = staffApiService.listUserIdByWxEnterpriseId(wxEnterpriseId, pageNum, 1000);
            List<StaffDTO> staffDTOS = staffDTOPageInfo.getResult();
            if (CollectionUtils.isEmpty(staffDTOS)) {
                break;
            }
            pageNum++;
            logger.info("useridToOpenuserid请求：corpid：{}", corpid);
            List<QywxNewUseridDTO> qywxNewUseridDTOS = qywxUserApiService.useridToOpenuserid(corpid, siteId, staffDTOS.stream().map(StaffDTO::getWxUserId).collect(Collectors.toList()));
            if (CollectionUtils.isNotEmpty(qywxNewUseridDTOS)) {
                for (QywxNewUseridDTO qywxNewUseridDTO : qywxNewUseridDTOS) {
                    staffApiService.updateOpenUserIdsByUserId(wxEnterpriseId, qywxNewUseridDTO.getUserid(), qywxNewUseridDTO.getOpen_userid());
                }
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 后台单个处理wx_user_id to wx_open_user_id
     *
     * @param corpid
     * @param staffId
     * @return
     */
    @RequestMapping("/useridToOpenuseridByStaffId")
    public HaobanResponse useridToOpenuseridByStaffId(String corpid, String staffId) {
        StaffDTO staffDTO = staffApiService.selectById(staffId);
        if (staffDTO == null) {
            return resultResponse(HaoBanErrCode.ERR_8);
        }
        String wxUserId = staffDTO.getWxUserId();
        String openUserid = getOpenUserid(wxUserId, corpid);
        staffApiService.updateOpenUserIdsByUserId(staffDTO.getWxEnterpriseId(), wxUserId, openUserid);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    /**
     * 获取openuserid
     *
     * @param userId
     * @param corpid
     * @return
     */
    private String getOpenUserid(String userId, String corpid) {
        WxApplicationDTO wxApplicationDTO = wxApplicationApiService.selectByCorpid(corpid);
        List<QywxNewUseridDTO> qywxNewUseridDTOS = qywxUserApiService.useridToOpenuserid(corpid, wxApplicationDTO.getSiteId(), Collections.singletonList(userId));
        if (CollectionUtils.isEmpty(qywxNewUseridDTOS)) {
            return "";
        }
        return qywxNewUseridDTOS.get(0).getOpen_userid();
    }

    /**
     * 代开发完成升级
     * @param corpid
     * @param agentid
     * @param openidType
     * @return
     */
    @RequestMapping("/finishOpenidMigration")
    public HaobanResponse finishOpenidMigration(String corpid, String agentid, String openidType) {
        String[] openidTypeSplit = openidType.split(",");
        boolean flag = qywxUserApiService.finishOpenidMigration(corpid, config.getCorpid(), agentid, openidTypeSplit);
        return resultResponse(HaoBanErrCode.ERR_1, flag);
    }

    /**
     * 转自建userid
     * @param corpid
     * @param secret
     * @param userIds
     * @return
     */
    @RequestMapping("/getSelfUseridToOpenuserid")
    public HaobanResponse getSelfUseridToOpenuserid(String corpid, String secret, String userIds) {
        String[] split = userIds.split(",");
        List<QywxNewUseridDTO> list = qywxUserApiService.getSelfUseridToOpenuserid(corpid, secret, Arrays.asList(split));
        return resultResponse(HaoBanErrCode.ERR_1, list);
    }
}
