package com.gic.haoban.manage.web.controller;

import com.gic.api.base.commons.Constant;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.base.api.common.pojo.dto.WebLoginDTO;
import com.gic.haoban.common.utils.AuthWebRequestUtil;
import com.gic.haoban.manage.api.dto.welcome.dto.FriendSettingDetailDTO;
import com.gic.haoban.manage.api.dto.welcome.dto.WelcomeDetailDTO;
import com.gic.haoban.manage.api.dto.welcome.dto.WelcomeListDTO;
import com.gic.haoban.manage.api.dto.welcome.qdto.SaveFriendSettingQDTO;
import com.gic.haoban.manage.api.dto.welcome.qdto.SaveWelcomeMediaQDTO;
import com.gic.haoban.manage.api.dto.welcome.qdto.SaveWelcomeQDTO;
import com.gic.haoban.manage.api.service.out.WelcomeApiService;
import com.gic.haoban.manage.web.qo.welcome.ListWelcomeQO;
import com.gic.haoban.manage.web.qo.welcome.SaveFriendSettingQO;
import com.gic.haoban.manage.web.qo.welcome.SaveWelcomeQO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * desc：欢迎语controller
 *
 * @author: YongEn
 * @date: 2022/2/8
 **/
@RestController
@RequestMapping("/welcome")
public class WelcomeController extends WebBaseController {

    @Autowired
    private WelcomeApiService welcomeApiService;


    /**
     * desc: 校验企业是否存在全员欢迎语
     *
     * @param wxEnterpriseId 微信企业id
     * @return : {@link RestResponse< Boolean>}
     * @author : YongEn
     * @date : 2022/3/9
     */
    @RequestMapping(value = "/exist-default-welcome")
    public RestResponse<Boolean> existDefaultWelcome(@RequestParam("wxEnterpriseId") String wxEnterpriseId) {
        ServiceResponse<Boolean> response = welcomeApiService.existDefaultWelcome(wxEnterpriseId);
        return RestResponse.successResult(response.getResult());
    }

    /**
     * desc: 保存欢迎语
     *
     * @param qo qo
     * @return : {@link RestResponse< Boolean>}
     * @author : YongEn
     * @date : 2022/2/11
     */
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public RestResponse<Boolean> saveWelcome(@RequestBody @Valid SaveWelcomeQO qo) {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        SaveWelcomeQDTO qdto = new SaveWelcomeQDTO();
        qdto.setWelcomeId(qo.getWelcomeId());
        qdto.setWxEnterpriseId(qo.getWxEnterpriseId());
        qdto.setTitle(qo.getTitle());
        qdto.setWelcomeContent(qo.getWelcomeContent());
        qdto.setOperatorId(login.getClerkId());
        qdto.setOperatorName(login.getClerkName());
        qdto.setDefaultWelcomeFlag(qo.getDefaultWelcomeFlag());
        qdto.setWelcomeMediaList(EntityUtil.changeEntityListNew(SaveWelcomeMediaQDTO.class, qo.getWelcomeMediaList()));
        qdto.setSuitDepartmentAddList(qo.getSuitDepartmentAddList());
        qdto.setSuitDepartmentDeleteList(qo.getSuitDepartmentDeleteList());
        qdto.setCreatorId(login.getClerkId());
        qdto.setCreatorName(login.getClerkFullName());
        qdto.setModifierId(login.getClerkId());
        qdto.setModifierName(login.getClerkFullName());
        ServiceResponse<Boolean> response = welcomeApiService.saveWelcome(qdto);
        return RestResponse.successResult(response.getResult());
    }

    /**
     * desc: 删除欢迎语
     *
     * @param welcomeId 欢迎语id
     * @return : {@link RestResponse< Boolean>}
     * @author : YongEn
     * @date : 2022/2/11
     */
    @RequestMapping(value = "/remove", method = RequestMethod.GET)
    public RestResponse<Boolean> removeWelcome(@RequestParam("welcomeId") String welcomeId) {
        if (StringUtils.isEmpty(welcomeId)) {
            RestResponse.failure(Constant.STRING_1, "欢迎语id为空", Boolean.FALSE);
        }
        ServiceResponse<Boolean> response = welcomeApiService.removeWelcome(welcomeId);
        return RestResponse.successResult(response.getResult());
    }

    /**
     * desc: 查询欢迎语列表
     *
     * @param qo qo
     * @return : {@link RestResponse< List< WelcomeListDTO>>}
     * @author : YongEn
     * @date : 2022/2/11
     */
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public RestResponse<List<WelcomeListDTO>> listWelcome(@RequestBody @Valid ListWelcomeQO qo) {
        ServiceResponse<List<WelcomeListDTO>> response = welcomeApiService.listWelcome(qo.getWxEnterpriseId(), qo.getTitle());
        return RestResponse.successResult(response.getResult());
    }

    /**
     * desc: 获取欢迎语
     *
     * @param welcomeId 欢迎语id
     * @return : {@link RestResponse< WelcomeDetailDTO>}
     * @author : YongEn
     * @date : 2022/2/11
     */
    @RequestMapping(value = "/get", method = RequestMethod.GET)
    public RestResponse<WelcomeDetailDTO> getWelcome(@RequestParam("welcomeId") String welcomeId,
                                                     @RequestParam("enterpriseId") String enterpriseId) {
        if (StringUtils.isEmpty(welcomeId)) {
            RestResponse.failure(Constant.STRING_1, "欢迎语id为空");
        }
        ServiceResponse<WelcomeDetailDTO> response = welcomeApiService.getWelcome(welcomeId, enterpriseId);
        return RestResponse.successResult(response.getResult());
    }

    /**
     * desc: 保存加好友配置
     *
     * @param qo qo
     * @return : {@link RestResponse< Boolean>}
     * @author : YongEn
     * @date : 2022/2/11
     * @deprecated 本次迭代即将删除
     */
    @RequestMapping(value = "/save/friend-setting", method = RequestMethod.POST)
    public RestResponse<Boolean> saveFriendSetting(@RequestBody @Valid SaveFriendSettingQO qo) {
        WebLoginDTO login = AuthWebRequestUtil.getLoginUser();
        SaveFriendSettingQDTO qdto = EntityUtil.changeEntityNew(SaveFriendSettingQDTO.class, qo);
        ServiceResponse<Boolean> response = welcomeApiService.saveFriendSetting(qdto);
        return RestResponse.successResult(response.getResult());
    }

    /**
     * desc: 获取加好友配置
     *
     * @param wxEnterpriseId 微信企业id
     * @return : {@link RestResponse< FriendSettingDetailDTO>}
     * @author : YongEn
     * @date : 2022/2/11
     * @deprecated 本次迭代即将删除
     */
    @RequestMapping(value = "/get/friend-setting", method = RequestMethod.GET)
    public RestResponse<FriendSettingDetailDTO> getFriendSetting(@RequestParam("wxEnterpriseId") String wxEnterpriseId) {
        if (StringUtils.isEmpty(wxEnterpriseId)) {
            RestResponse.failure(Constant.STRING_1, "微信企业id为空", Boolean.FALSE);
        }
        ServiceResponse<FriendSettingDetailDTO> response = welcomeApiService.getFriendSetting(wxEnterpriseId);
        return RestResponse.successResult(response.getResult());
    }

    /**
     * desc: 保存默认欢迎语
     *
     * @param wxEnterpriseId 微信企业id
     * @return : {@link RestResponse< Boolean>}
     * @author : YongEn
     * @date : 2022/2/25
     */
    @RequestMapping(value = "/save-default-welcome", method = RequestMethod.GET)
    public RestResponse<Boolean> saveDefaultWelcome(@RequestParam("wxEnterpriseId") String wxEnterpriseId) {
        ServiceResponse<Boolean> response = welcomeApiService.saveDefaultWelcome(wxEnterpriseId);
        return RestResponse.successResult(response.getResult());
    }

    /**
     * desc: 刷新欢迎语媒体
     *
     * @return : {@link RestResponse< Boolean>}
     * @author : YongEn
     * @date : 2022/3/11
     */
    @RequestMapping(value = "/flush-welcome-media", method = RequestMethod.GET)
    public RestResponse<Boolean> flushWelcomeMedia() {
        ServiceResponse<Boolean> response = welcomeApiService.flushWelcomeMedia();
        return RestResponse.successResult(response.getResult());
    }

}
